@@ -25,14 +25,17 @@ import numpy
 from collada.common import DaeObject, E, tag
 from collada.common import DaeIncompleteError, DaeBrokenRefError, \
         DaeMalformedError, DaeUnsupportedError
-from collada.util import falmostEqual, StringIO
+from collada.util import falmostEqual, BytesIO
 from collada.xmlutil import etree as ElementTree
 
 try:
-    import Image as pil
+    from PIL import Image as pil
 except:
     pil = None
 
+# internally used constant
+_FAILED = 'failed'
+
 
 class DaeMissingSampler2D(Exception):
     """Raised when a <texture> tag references a texture without a sampler."""
@@ -88,41 +91,41 @@ class CImage(DaeObject):
         return self._data
 
     def getImage(self):
-        if pil is None or self._pilimage == 'failed':
+        if pil is None or self._pilimage is _FAILED:
             return None
         if self._pilimage:
             return self._pilimage
         else:
             data = self.getData()
             if not data:
-                self._pilimage = 'failed'
+                self._pilimage = _FAILED
                 return None
             try:
-                self._pilimage = pil.open( StringIO(data) )
+                self._pilimage = pil.open( BytesIO(data) )
                 self._pilimage.load()
             except IOError as ex:
-                self._pilimage = 'failed'
+                self._pilimage = _FAILED
                 return None
             return self._pilimage
 
     def getUintArray(self):
-        if self._uintarray == 'failed': return None
-        if self._uintarray != None: return self._uintarray
+        if self._uintarray is _FAILED: return None
+        if self._uintarray is not None: return self._uintarray
         img = self.getImage()
         if not img:
-            self._uintarray = 'failed'
+            self._uintarray = _FAILED
             return None
         nchan = len(img.mode)
-        self._uintarray = numpy.fromstring(img.tostring(), dtype=numpy.uint8)
+        self._uintarray = numpy.fromstring(img.tobytes(), dtype=numpy.uint8)
         self._uintarray.shape = (img.size[1], img.size[0], nchan)
         return self._uintarray
 
     def getFloatArray(self):
-        if self._floatarray == 'failed': return None
-        if self._floatarray != None: return self._floatarray
+        if self._floatarray is _FAILED: return None
+        if self._floatarray is not None: return self._floatarray
         array = self.getUintArray()
         if array is None:
-            self._floatarray = 'failed'
+            self._floatarray = _FAILED
             return None
         self._floatarray = numpy.asarray( array, dtype=numpy.float32)
         self._floatarray *= 1.0/255.0
@@ -371,7 +374,7 @@ class Map(DaeObject):
         sampler = localscope.get(samplerid)
         #Check for the sampler ID as the texture ID because some exporters suck
         if sampler is None:
-            for s2d in localscope.itervalues():
+            for s2d in localscope.values():
                 if type(s2d) is Sampler2D:
                     if s2d.surface.image.id == samplerid:
                         sampler = s2d
@@ -546,7 +549,27 @@ class Effect(DaeObject):
             self.xmlnode = E.effect(
                 E.profile_COMMON(*effect_nodes)
             , id=self.id, name=self.id)
+            
+    @staticmethod
+    def getEffectParameters(collada, parentnode, localscope, params):
 
+        for paramnode in parentnode.findall( tag('newparam') ):
+            if paramnode.find( tag('surface') ) is not None:
+                param = Surface.load(collada, localscope, paramnode)
+                params.append(param)
+                localscope[param.id] = param
+            elif paramnode.find( tag('sampler2D') ) is not None:                
+                param = Sampler2D.load(collada, localscope, paramnode)
+                params.append(param)
+                localscope[param.id] = param
+            else:
+                floatnode = paramnode.find( tag('float') )
+                if floatnode is None: floatnode = paramnode.find( tag('float2') )
+                if floatnode is None: floatnode = paramnode.find( tag('float3') )
+                if floatnode is None: floatnode = paramnode.find( tag('float4') )
+                paramid = paramnode.get('sid')
+                if floatnode is not None and paramid is not None and len(paramid) > 0 and floatnode.text is not None:
+                    localscope[paramid] = [float(v) for v in floatnode.text.split()]
 
     @staticmethod
     def load(collada, localscope, node):
@@ -569,24 +592,12 @@ class Effect(DaeObject):
                 uniquenum += 1
             collada.images.append(local_image)
 
-        for paramnode in profilenode.findall( tag('newparam') ):
-            if paramnode.find( tag('surface') ) is not None:
-                param = Surface.load(collada, localscope, paramnode)
-                params.append(param)
-                localscope[param.id] = param
-            elif paramnode.find( tag('sampler2D') ) is not None:
-                param = Sampler2D.load(collada, localscope, paramnode)
-                params.append(param)
-                localscope[param.id] = param
-            else:
-                floatnode = paramnode.find( tag('float') )
-                if floatnode is None: floatnode = paramnode.find( tag('float2') )
-                if floatnode is None: floatnode = paramnode.find( tag('float3') )
-                if floatnode is None: floatnode = paramnode.find( tag('float4') )
-                paramid = paramnode.get('sid')
-                if floatnode is not None and paramid is not None and len(paramid) > 0 and floatnode.text is not None:
-                    localscope[paramid] = [float(v) for v in floatnode.text.split()]
+        Effect.getEffectParameters(collada, profilenode, localscope, params)
+        
         tecnode = profilenode.find( tag('technique') )
+        
+        Effect.getEffectParameters(collada, tecnode, localscope, params)
+        
         shadnode = None
         for shad in Effect.shaders:
             shadnode = tecnode.find(tag(shad))
@@ -618,7 +629,7 @@ class Effect(DaeObject):
                 except DaeUnsupportedError as ex:
                     props[key] = None
                     collada.handleError(ex) # Give the chance to ignore error and load the rest
-                
+
                 if key == 'transparent' and key in props and props[key] is not None:
                     opaque_mode = pnode.get('opaque')
                     if opaque_mode is not None and opaque_mode == OPAQUE_MODE.RGB_ZERO:
@@ -873,4 +884,3 @@ class Material(DaeObject):
 
     def __repr__(self):
         return str(self)
-
