@@ -184,27 +184,43 @@ class TriangleSet(primitive.Primitive):
 
     def __getitem__(self, i):
         v = self._vertex[ self._vertex_index[i] ]
-        n = self._normal[ self._normal_index[i] ]
+        n = self._normal[ self._normal_index[i] ] if self._normal is not None else None
         uvindices = []
         uv = []
         for j, uvindex in enumerate(self._texcoord_indexset):
             uvindices.append( uvindex[i] )
             uv.append( self._texcoordset[j][ uvindex[i] ] )
-        return Triangle(self._vertex_index[i], v, self._normal_index[i], n, uvindices, uv, self.material)
+        return Triangle(self._vertex_index[i], v, self._normal_index[i] if self._normal_index is not None else 0, n, uvindices, uv, self.material)
 
     @staticmethod
     def load( collada, localscope, node ):
-        indexnode = node.find(tag('p'))
-        if indexnode is None: raise DaeIncompleteError('Missing index in triangle set')
+        indexnodes = node.findall(tag('p'))
+        if not indexnodes: raise DaeIncompleteError('Missing index in triangle set')
 
         source_array = primitive.Primitive._getInputs(collada, localscope, node.findall(tag('input')))
 
-        try:
+        def parse_p(indexnode):
             if indexnode.text is None:
                 index = numpy.array([], dtype=numpy.int32)
             else:
                 index = numpy.fromstring(indexnode.text, dtype=numpy.int32, sep=' ')
             index[numpy.isnan(index)] = 0
+            return index
+
+        indexlist = []
+        extendfunc = _indexExtendFunctions[node.tag]
+        max_offset = max(input[0] for input_type_array in source_array.values()
+                         for input in input_type_array)
+
+        try:
+            for indexnode in indexnodes:
+                index = parse_p(indexnode)
+                if extendfunc is None:
+                    break
+
+                extendfunc(indexlist, index.reshape((-1, max_offset + 1)))
+            else:
+                index = numpy.concatenate(indexlist)
         except:
             raise DaeMalformedError('Corrupted index in triangleset')
 
@@ -397,3 +413,31 @@ class BoundTriangleSet(primitive.BoundPrimitive):
     def __repr__(self):
         return str(self)
 
+
+def _extendFromStrip(indexlist, index):
+    """Convert triangle strip indices to triangle indices
+
+    :param list indexlist:
+      list to append 1-dimensional index arrays to
+    :param numpy.ndarray index:
+      (#vertices, #inputs) shaped index array
+    """
+    cw_ = numpy.array([index[0:-2:2], index[1:-1:2], index[2::2]])
+    ccw = numpy.array([index[2:-1:2], index[1:-2:2], index[3::2]])
+    indexlist.append(cw_.swapaxes(0, 1).ravel())
+    indexlist.append(ccw.swapaxes(0, 1).ravel())
+
+def _extendFromFan(indexlist, index):
+    """Convert triangle fan indices to triangle indices
+    """
+    c = numpy.concatenate((
+        numpy.repeat(index[:1], len(index) - 2, 0),
+        index[1:-1],
+        index[2:]), 1)
+    indexlist.append(c.reshape(-1))
+
+_indexExtendFunctions = {
+    tag('tristrips'): _extendFromStrip,
+    tag('trifans'): _extendFromFan,
+    tag('triangles'): None,
+}
